;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Revival "Block"
;;   by TheBiob
;; Description: If the player is dead and touches this block he will get revived
;;
;; Uses first extra bit: NO
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;
;   DEFINES   ;
;;;;;;;;;;;;;;;

	!tile	= $00		; What tile is used to display
	!properties	= $30	; YXPPCCCT properties of that tile (please setting palette from cfg file)
	
	!invul	= $30		; How long the player will be invulnerable for
	
	!reviveSound	= $02	; \ Which sound to play. See http://www.smwcentral.net/?p=viewthread&t=6665 for more information
	!sndAddress		= $1DFC	; /
	
	!lives	= $01	; How many lives the player looses when being revived (Set to 0 if you don't want lives removed

;;;;;;;;;;;;;;;
; Sprite INIT ;
;;;;;;;;;;;;;;;

	print "INIT ",pc
		LDA $0DDA	; \ Store backup of music register to $1602,x
		CMP #$FF	; |
		BEQ +		; |
		AND #$3F	; |
		STA $1602,x	; /
+	RTL
	
;;;;;;;;;;;;;;;
; Sprite MAIN ;
;;;;;;;;;;;;;;;

	print "MAIN ",pc
		PHB : PHK : PLB
			JSR SpriteMain
		PLB
	RTL

SpriteMain:
	JSR GFX		; > Run graphic routine
	LDY $9D
	BNE +
	JSL $01B44F	; > Invisible solid block routine
+	LDA $157C,x	; \ Check if music should restart
	BEQ +		; |
	LDA $1540,x	; |
	BNE ++		; |
	LDA $1602,x	; |
	STA $1DFB	; |
	STZ $157C,x	; /
	BRA ++
+	LDY #$02
	JSR SubOffScreen; Handle sprite off screen/but only when music isn't about to restart (To prevent the sprite accidentally despawning before the music can restart)
++	LDA $71		; \ Check if player is dying
	CMP #$09	; |
	BNE .return	; /
	JSL $03B664	; \ Get player clipping
	JSL $03B69F	; | Get sprite clipping
	JSL $03B72B	; / Check for contact
	BCC .return	; > Return if no contact
	LDA #!lives	; \ Remove lives
	BEQ +		; |
	LDA $0DBE	; |
	CMP #!lives	; |
	BCC .return	; |
	SBC #!lives	; |
	STA $0DBE	; /
+	STZ $71		; > Set mode to "move freely"
	LDA #!invul	; \ Set flashing invulnerability timer
	STA $1497	; /
	LDA #$80	; \ Play music fadeout
	STA $1DFB	; /
	LDA #$40	; \ Set timer to restart actual game music again
	STA $1540,x	; /
	LDA #$01	; \ Set restart music flag
	STA $157C,x	; /
	LDA #!reviveSound	; \ Play revive sound
	STA !sndAddress		; /
.return
RTS

GFX:
	JSR GetDrawInfo
	
	LDA $00			; x position
	STA $0300,y
	LDA $01			; y position
	STA $0301,y
	LDA #!tile 		; tile
	STA $0302,y
	LDA $15F6,x		; palette
	ORA #!properties; properties
	STA $0303,y
	
	LDY #$02		; 00 = 8x8, 02 = 16x16, FF = Don't set tile size
	LDA #$00		; How many tile have been drawn - 1
	JSL $01B7B3		; reserve
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GetDrawInfo - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GetDrawInfo:
		LDA $14E0,x
		XBA
		LDA $E4,x
		REP #$20
		SEC
		SBC $1A
		STA $00
		CLC
		ADC.w #$0040
		CMP.w #$0180
		SEP #$20
		LDA $01
		BEQ +
		LDA #$01
+		STA $15A0,x
		TDC
		ROL A
		STA $15C4,x
		BNE .Invalid

		LDA $14D4,x
		XBA
		LDA $1662,x
		AND #$20
		BEQ .CheckOnce
.CheckTwice	LDA $D8,x
		REP #$21
		ADC.w #$001C
		SEC
		SBC $1C
		SEP #$20
		LDA $14D4,x
		XBA
		BEQ .CheckOnce
		LDA #$02
.CheckOnce	STA $186C,x
		LDA $D8,x
		REP #$21
		ADC.w #$000C
		SEC
		SBC $1C
		SEP #$21
		SBC #$0C
		STA $01
		XBA
		BEQ .OnScreenY
		INC $186C,x
.OnScreenY	LDY $15EA,x
		RTS
		
.Invalid	PLA
		PLA
		RTS
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SubOffScreen - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Argument
; Y : type (#$00=X0, #$02=X1 ... #$0E=X7)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
SubOffScreen:	LDA $15A0,x
		ORA $186C,x
		BEQ .Return2
		LDA $167A,x
		AND #$04
		BNE .Return2
		LDA $5B
		LSR A
		BCS .VerticalLevel
		LDA $D8,x
		ADC #$50
		LDA $14D4,x
		ADC #$00
		CMP #$02
		BCS .EraseSprite
		LDA $14E0,x
		XBA
		LDA $E4,x
		REP #$21
		ADC .AddTable,y
		SEC
		SBC $1A
		CMP .CmpTable,y
.Common		SEP #$20
		BCC .Return2
.EraseSprite	LDA $14C8,x
		CMP #$08
		BCC .KillSprite
		LDY $161A,x
		CPY #$FF
		BEQ .KillSprite
		LDA $1938,y
		AND #$FE
		STA $1938,y
.KillSprite	STZ $14C8,x
.Return2	RTS

.VerticalLevel	LDA $13
		LSR A
		BCS .CheckY
		LDA $14E0,x
		XBA
		LDA $E4,x
		REP #$21
		ADC.w #$0040
		CMP.w #$0280
		BRA .Common
.CheckY		LDA $14D4,x
		XBA
		LDA $D8,x
		REP #$20
		SBC $1C
		CLC
		ADC.w #$0070
		CMP.w #$01D0
		BRA .Common

.AddTable	dw $0040,$0040,$0010,$0070
		dw $0090,$0050,$0080,$FFC0
.CmpTable	dw $0170,$01E0,$01B0,$01D0
		dw $0230,$01B0,$0220,$0160